@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2016 NIWA
+# Copyright (C) 2008-2017 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -45,20 +45,20 @@ class PBSHandler(object):
 
     def format_directives(self, job_conf):
         """Format the job directives for a job file."""
-        job_file_path = job_conf["job file path"].replace(r"$HOME/", "")
+        job_file_path = job_conf["job_file_path"].replace(r"$HOME/", "")
         directives = job_conf["directives"].__class__()  # an ordereddict
 
-        directives["-N"] = job_conf["task id"] + "." + job_conf["suite name"]
-        job_name_len_max = job_conf['batch system conf'].get(
+        directives["-N"] = job_conf["task_id"] + "." + job_conf["suite_name"]
+        job_name_len_max = job_conf['batch_system_conf'].get(
             "job name length maximum", self.JOB_NAME_LEN_MAX)
         if job_name_len_max:
             directives["-N"] = directives["-N"][0:job_name_len_max]
 
         directives["-o"] = job_file_path + ".out"
         directives["-e"] = job_file_path + ".err"
-        if (job_conf["execution time limit"] and
+        if (job_conf["execution_time_limit"] and
                 directives.get("-l walltime") is None):
-            directives["-l walltime"] = "%d" % job_conf["execution time limit"]
+            directives["-l walltime"] = "%d" % job_conf["execution_time_limit"]
         for key, value in job_conf["directives"].items():
             directives[key] = value
         lines = []
