@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2016 NIWA
+# Copyright (C) 2008-2017 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -36,13 +36,8 @@ from cylc.cfgspec.utils import (
     coerce_interval, coerce_interval_list, DurationFloat)
 from cylc.network import PRIVILEGE_LEVELS
 
-coercers['interval_seconds'] = lambda *args: coerce_interval(
-    *args, check_syntax_version=False)
-coercers['interval_minutes'] = lambda *args: coerce_interval(
-    *args, back_comp_unit_factor=60, check_syntax_version=False)
-coercers['interval_minutes_list'] = lambda *args: coerce_interval_list(
-    *args, back_comp_unit_factor=60, check_syntax_version=False)
-
+coercers['interval'] = coerce_interval
+coercers['interval_list'] = coerce_interval_list
 
 SPEC = {
     'process pool size': vdr(vtype='integer', default=4),
@@ -53,24 +48,22 @@ SPEC = {
     'enable run directory housekeeping': vdr(vtype='boolean', default=False),
     'run directory rolling archive length': vdr(
         vtype='integer', vmin=0, default=2),
-    'submission polling intervals': vdr(
-        vtype='interval_minutes_list', default=[]),
-    'execution polling intervals': vdr(
-        vtype='interval_minutes_list', default=[]),
-
     'task host select command timeout': vdr(
-        vtype='interval_seconds', default=DurationFloat(10)),
+        vtype='interval', default=DurationFloat(10)),
     'task messaging': {
         'retry interval': vdr(
-            vtype='interval_seconds', default=DurationFloat(5)),
+            vtype='interval', default=DurationFloat(5)),
         'maximum number of tries': vdr(vtype='integer', vmin=1, default=7),
         'connection timeout': vdr(
-            vtype='interval_seconds', default=DurationFloat(30)),
-
+            vtype='interval', default=DurationFloat(30)),
     },
 
     'cylc': {
         'UTC mode': vdr(vtype='boolean', default=False),
+        'health check interval': vdr(
+            vtype='interval', default=DurationFloat(600)),
+        'task event mail interval': vdr(
+            vtype='interval', default=DurationFloat(300)),
         'events': {
             'handlers': vdr(vtype='string_list', default=[]),
             'handler events': vdr(vtype='string_list', default=[]),
@@ -84,8 +77,8 @@ SPEC = {
             'inactivity handler': vdr(vtype='string_list', default=[]),
             'shutdown handler': vdr(vtype='string_list', default=[]),
             'stalled handler': vdr(vtype='string_list', default=[]),
-            'timeout': vdr(vtype='interval_minutes'),
-            'inactivity': vdr(vtype='interval_minutes'),
+            'timeout': vdr(vtype='interval'),
+            'inactivity': vdr(vtype='interval'),
             'abort on timeout': vdr(vtype='boolean', default=False),
             'abort on inactivity': vdr(vtype='boolean', default=False),
             'abort on stalled': vdr(vtype='boolean', default=False),
@@ -102,19 +95,20 @@ SPEC = {
     'documentation': {
         'files': {
             'html index': vdr(
-                vtype='string', default="$CYLC_DIR/doc/index.html"),
+                vtype='string', default="$CYLC_DIR/doc/install/index.html"),
             'pdf user guide': vdr(
-                vtype='string', default="$CYLC_DIR/doc/pdf/cug-pdf.pdf"),
+                vtype='string',
+                default="$CYLC_DIR/doc/install/cylc-user-guide.pdf"),
             'multi-page html user guide': vdr(
                 vtype='string',
-                default="$CYLC_DIR/doc/html/multi/cug-html.html"),
+                default="$CYLC_DIR/doc/install/html/multi/cug-html.html"),
             'single-page html user guide': vdr(
                 vtype='string',
-                default="$CYLC_DIR/doc/html/single/cug-html.html"),
+                default="$CYLC_DIR/doc/install/html/single/cug-html.html"),
         },
         'urls': {
             'internet homepage': vdr(
-                vtype='string', default="http://cylc.github.com/cylc/"),
+                vtype='string', default="http://cylc.github.io/cylc/"),
             'local index': vdr(vtype='string', default=None),
         },
     },
@@ -128,10 +122,13 @@ SPEC = {
         'gui': vdr(vtype='string', default="gvim -f"),
     },
 
-    'pyro': {
-        'base port': vdr(vtype='integer', default=7766),
+    'communication': {
+        'method': vdr(vtype='string', default="https",
+                      options=["https"]),
+        'base port': vdr(vtype='integer', default=43001),
         'maximum number of ports': vdr(vtype='integer', default=100),
-        'ports directory': vdr(vtype='string', default="$HOME/.cylc/ports/"),
+        'proxies on': vdr(vtype='boolean', default=False),
+        'options': vdr(vtype='string_list', default=[]),
     },
 
     'monitor': {
@@ -146,11 +143,15 @@ SPEC = {
             'work directory': vdr(vtype='string', default="$HOME/cylc-run"),
             'task communication method': vdr(
                 vtype='string',
-                options=["pyro", "ssh", "poll"], default="pyro"),
-            'remote copy template': vdr(
+                options=["default", "ssh", "poll"], default="default"),
+            'submission polling intervals': vdr(
+                vtype='interval_list', default=[]),
+            'execution polling intervals': vdr(
+                vtype='interval_list', default=[]),
+            'scp command': vdr(
                 vtype='string',
                 default='scp -oBatchMode=yes -oConnectTimeout=10'),
-            'remote shell template': vdr(
+            'ssh command': vdr(
                 vtype='string',
                 default='ssh -oBatchMode=yes -oConnectTimeout=10'),
             'use login shell': vdr(vtype='boolean', default=True),
@@ -163,9 +164,9 @@ SPEC = {
                 vtype='string', default='rsync -a'),
             'retrieve job logs max size': vdr(vtype='string'),
             'retrieve job logs retry delays': vdr(
-                vtype='interval_minutes_list', default=[]),
+                vtype='interval_list', default=[]),
             'task event handler retry delays': vdr(
-                vtype='interval_minutes_list', default=[]),
+                vtype='interval_list', default=[]),
             'local tail command template': vdr(
                 vtype='string', default="tail -n +1 -F %(filename)s"),
             'remote tail command template': vdr(
@@ -190,7 +191,7 @@ SPEC = {
                     'out viewer': vdr(vtype='string'),
                     'job name length maximum': vdr(vtype='integer'),
                     'execution time limit polling intervals': vdr(
-                        vtype='interval_minutes_list', default=[]),
+                        vtype='interval_list', default=[]),
                 },
             },
         },
@@ -198,9 +199,13 @@ SPEC = {
             'run directory': vdr(vtype='string'),
             'work directory': vdr(vtype='string'),
             'task communication method': vdr(
-                vtype='string', options=["pyro", "ssh", "poll"]),
-            'remote copy template': vdr(vtype='string'),
-            'remote shell template': vdr(vtype='string'),
+                vtype='string', options=["default", "ssh", "poll"]),
+            'submission polling intervals': vdr(
+                vtype='interval_list', default=[]),
+            'execution polling intervals': vdr(
+                vtype='interval_list', default=[]),
+            'scp command': vdr(vtype='string'),
+            'ssh command': vdr(vtype='string'),
             'use login shell': vdr(vtype='boolean', default=None),
             'cylc executable': vdr(vtype='string'),
             'global init-script': vdr(vtype='string'),
@@ -210,9 +215,9 @@ SPEC = {
             'retrieve job logs command': vdr(vtype='string'),
             'retrieve job logs max size': vdr(vtype='string'),
             'retrieve job logs retry delays': vdr(
-                vtype='interval_minutes_list'),
+                vtype='interval_list'),
             'task event handler retry delays': vdr(
-                vtype='interval_minutes_list'),
+                vtype='interval_list'),
             'local tail command template': vdr(vtype='string'),
             'remote tail command template': vdr(vtype='string'),
             'batch systems': {
@@ -223,24 +228,24 @@ SPEC = {
                     'err viewer': vdr(vtype='string'),
                     'job name length maximum': vdr(vtype='integer'),
                     'execution time limit polling intervals': vdr(
-                        vtype='interval_minutes_list'),
+                        vtype='interval_list'),
                 },
             },
         },
     },
 
     'task events': {
-        'execution timeout': vdr(vtype='interval_minutes'),
+        'execution timeout': vdr(vtype='interval'),
         'handlers': vdr(vtype='string_list', default=[]),
         'handler events': vdr(vtype='string_list', default=[]),
-        'handler retry delays': vdr(vtype='interval_minutes_list'),
+        'handler retry delays': vdr(vtype='interval_list'),
         'mail events': vdr(vtype='string_list', default=[]),
         'mail from': vdr(vtype='string'),
-        'mail retry delays': vdr(vtype='interval_minutes_list', default=[]),
+        'mail retry delays': vdr(vtype='interval_list', default=[]),
         'mail smtp': vdr(vtype='string'),
         'mail to': vdr(vtype='string'),
         'reset timer': vdr(vtype='boolean', default=False),
-        'submission timeout': vdr(vtype='interval_minutes'),
+        'submission timeout': vdr(vtype='interval'),
     },
 
     'test battery': {
@@ -276,14 +281,6 @@ SPEC = {
             vtype='string',
             options=PRIVILEGE_LEVELS[:PRIVILEGE_LEVELS.index('shutdown') + 1],
             default="state-totals"),
-        'hashes': vdr(
-            vtype='string_list',
-            options=['md5', 'sha1', 'sha256', 'sha512'],
-            default=['sha256', 'md5']),
-        'scan hash': vdr(
-            vtype='string',
-            options=['md5', 'sha1', 'sha256', 'sha512'],
-            default='md5'),
     },
 }
 
@@ -341,6 +338,47 @@ def upg(cfg, descr):
     for key in SPEC['cylc']['events']:
         u.deprecate(
             '6.11.0', ['cylc', 'event hooks', key], ['cylc', 'events', key])
+    u.obsolete(
+        '7.0.0',
+        ['pyro', 'base port']
+    )
+    u.obsolete(
+        '7.0.0',
+        ['pyro', 'maximum number of ports'],
+        ['communication', 'maximum number of ports']
+    )
+    u.obsolete(
+        '7.0.0',
+        ['pyro', 'ports directory'],
+    )
+    u.obsolete(
+        '7.0.0',
+        ['pyro']
+    )
+    u.obsolete(
+        '7.0.0',
+        ['authentication', 'hashes']
+    )
+    u.obsolete(
+        '7.0.0',
+        ['authentication', 'scan hash']
+    )
+    u.deprecate(
+        '7.0.0',
+        ['execution polling intervals'],
+        ['hosts', 'localhost', 'execution polling intervals'])
+    u.deprecate(
+        '7.0.0',
+        ['submission polling intervals'],
+        ['hosts', 'localhost', 'submission polling intervals'])
+    u.deprecate(
+        '7.3.1',
+        ['hosts', '__MANY__', 'remote shell template'],
+        ['hosts', '__MANY__', 'ssh command'])
+    u.deprecate(
+        '7.3.1',
+        ['hosts', '__MANY__', 'remote copy template'],
+        ['hosts', '__MANY__', 'scp command'])
     u.upgrade()
 
 
@@ -431,6 +469,9 @@ class GlobalConfig(config):
         elif item == 'suite config log directory':
             value = os.path.join(srdir, 'log', 'suiterc')
 
+        elif item == 'suite work root':
+            value = swdir
+
         elif item == 'suite work directory':
             value = os.path.join(swdir, 'work')
 
@@ -545,10 +586,6 @@ class GlobalConfig(config):
         if value:
             self.create_directory(value, item)
 
-        item = '[pyro]ports directory'
-        value = cfg['pyro']['ports directory']
-        self.create_directory(value, item)
-
     def get_tmpdir(self):
         """Make a new temporary directory and arrange for it to be
         deleted automatically when we're finished with it. Call this
@@ -584,7 +621,10 @@ class GlobalConfig(config):
             if host == 'localhost':
                 continue
             for item, value in cfg['hosts'][host].items():
-                newvalue = value or cfg['hosts']['localhost'][item]
+                if value is None:
+                    newvalue = cfg['hosts']['localhost'][item]
+                else:
+                    newvalue = value
                 if newvalue and 'directory' in item:
                     # replace local home dir with $HOME for evaluation on other
                     # host
@@ -595,9 +635,6 @@ class GlobalConfig(config):
         for key, val in cfg['documentation']['files'].items():
             cfg['documentation']['files'][key] = expandvars(val)
 
-        cfg['pyro']['ports directory'] = expandvars(
-            cfg['pyro']['ports directory'])
-
         for key, val in cfg['hosts']['localhost'].items():
             if val and 'directory' in key:
                 cfg['hosts']['localhost'][key] = expandvars(val)
