@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2016 NIWA
+# Copyright (C) 2008-2017 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,20 +18,26 @@
 
 """This module provides base classes for cycling data objects."""
 
+import unittest
+
+from abc import ABCMeta, abstractmethod, abstractproperty
+
 
 def parse_exclusion(expr):
     count = expr.count('!')
     if count == 0:
         return expr, None
     elif count > 1:
-        raise Exception("'%s': only one exclusion per expression "
+        raise Exception("'%s': only one set of exclusions per expression "
                         "permitted" % expr)
     else:
-        remainder, exclusion = expr.split('!')
-        if '/' in exclusion:
+        remainder, exclusions = expr.split('!')
+        if '/' in exclusions:
             raise Exception("'%s': exclusion must be at the end of the "
                             "expression" % expr)
-        return remainder.strip(), exclusion.strip()
+        exclusions = exclusions.translate(None, ' ()')
+        exclusions = exclusions.split(',')
+        return remainder.strip(), exclusions
 
 
 class CyclerTypeError(TypeError):
@@ -77,7 +83,7 @@ class SequenceDegenerateError(Exception):
 
 class PointBase(object):
 
-    """The base class for single points in a cycler sequence.
+    """The abstract base class for single points in a cycler sequence.
 
     Points should be based around a string value.
 
@@ -90,27 +96,38 @@ class PointBase(object):
     method to reprocess their value into a standard form.
 
     """
+    __metaclass__ = ABCMeta
 
-    TYPE = None
-    TYPE_SORT_KEY = None
+    _TYPE = None
+    _TYPE_SORT_KEY = None
+
+    @abstractproperty
+    def TYPE(self):
+        return self._TYPE
+
+    @abstractproperty
+    def TYPE_SORT_KEY(self):
+        return self._TYPE_SORT_KEY
 
     def __init__(self, value):
         if not isinstance(value, basestring):
             raise TypeError(type(value))
         self.value = value
 
+    @abstractmethod
     def add(self, other):
         """Add other (interval) to self, returning a point."""
-        raise NotImplementedError()
+        pass
 
     def cmp_(self, other):
         """Compare self to other point, returning a 'cmp'-like result."""
-        raise NotImplementedError()
+        pass
 
     def standardise(self):
         """Format self.value into a standard representation and check it."""
         return self
 
+    @abstractmethod
     def sub(self, other):
         """Subtract other (interval or point), returning a point or interval.
 
@@ -121,16 +138,18 @@ class PointBase(object):
          IntervalBase-derived object)
 
         """
-        raise NotImplementedError()
+        pass
 
     def __str__(self):
         # Stringify.
         return self.value
 
+    __repr__ = __str__
+
     def __cmp__(self, other):
         # Compare to other point.
-        if not other:
-            return False
+        if other is None:
+            return -1
         if self.TYPE != other.TYPE:
             return cmp(self.TYPE_SORT_KEY, other.TYPE_SORT_KEY)
         if self.value == other.value:
@@ -176,36 +195,51 @@ class IntervalBase(object):
     method to reprocess their value into a standard form.
 
     """
+    __metaclass__ = ABCMeta
 
-    TYPE = None
-    TYPE_SORT_KEY = None
+    _TYPE = None
+    _TYPE_SORT_KEY = None
+
+    @abstractproperty
+    def TYPE(self):
+        return self._TYPE
+
+    @abstractproperty
+    def TYPE_SORT_KEY(self):
+        return self._TYPE_SORT_KEY
 
     @classmethod
+    @abstractmethod
     def get_null(cls):
         """Return a null interval."""
-        raise NotImplementedError()
+        pass
 
+    @abstractmethod
     def get_inferred_child(self, string):
         """For a given string, infer the offset given my instance units."""
-        raise NotImplementedError()
+        pass
 
+    @abstractmethod
     def __abs__(self):
         # Return an interval with absolute values for all properties.
-        raise NotImplementedError()
+        pass
 
+    @abstractmethod
     def __mul__(self, factor):
         # Return an interval with all properties multiplied by factor.
-        raise NotImplementedError()
+        pass
 
+    @abstractmethod
     def __nonzero__(self):
         # Return True if the interval has any non-zero properties.
-        raise NotImplementedError()
+        pass
 
     def __init__(self, value):
         if not isinstance(value, basestring):
             raise TypeError(type(value))
         self.value = value
 
+    @abstractmethod
     def add(self, other):
         """Add other to self, returning a Point or Interval.
 
@@ -216,19 +250,21 @@ class IntervalBase(object):
          IntervalBase-derived object)
 
         """
-        raise NotImplementedError()
+        pass
 
+    @abstractmethod
     def cmp_(self, other):
         """Compare self to other (interval), returning a 'cmp'-like result."""
-        raise NotImplementedError()
+        pass
 
     def standardise(self):
         """Format self.value into a standard representation."""
         return self
 
+    @abstractmethod
     def sub(self, other):
         """Subtract other (interval) from self; return an interval."""
-        raise NotImplementedError()
+        pass
 
     def is_null(self):
         return (self == self.get_null())
@@ -264,7 +300,7 @@ class IntervalBase(object):
 
 class SequenceBase(object):
 
-    """The base class for cycler sequences.
+    """The abstract base class for cycler sequences.
 
     Subclasses should accept a sequence-specific string, a
     start context string, and a stop context string as
@@ -283,68 +319,107 @@ class SequenceBase(object):
     is equal to another (represents the same set of points).
 
     """
+    __metaclass__ = ABCMeta
 
-    TYPE = None
-    TYPE_SORT_KEY = None
+    _TYPE = None
+    _TYPE_SORT_KEY = None
+
+    @abstractproperty
+    def TYPE(self):
+        return self._TYPE
+
+    @abstractproperty
+    def TYPE_SORT_KEY(self):
+        return self._TYPE_SORT_KEY
 
     @classmethod
+    @abstractmethod  # Note: stacked decorator not strictly enforced in Py2.x
     def get_async_expr(cls, start_point=0):
         """Express a one-off sequence at the initial cycle point."""
-        raise NotImplementedError()
+        pass
 
+    @abstractmethod
     def __init__(self, sequence_string, context_start, context_stop=None):
         """Parse sequence string according to context point strings."""
         pass
 
+    @abstractmethod
     def get_interval(self):
         """Return the cycling interval of this sequence."""
-        raise NotImplementedError()
+        pass
 
+    @abstractmethod
     def get_offset(self):
         """Deprecated: return the offset used for this sequence."""
-        raise NotImplementedError()
+        pass
 
+    @abstractmethod
     def set_offset(self, i_offset):
         """Deprecated: alter state to offset the entire sequence."""
-        raise NotImplementedError()
+        pass
 
+    @abstractmethod
     def is_on_sequence(self, point):
         """Is point on-sequence, disregarding bounds?"""
-        raise NotImplementedError()
+        pass
 
     def _get_point_in_bounds(self, point):
         """Return point, or None if out of bounds."""
-        raise NotImplementedError()
+        raise NotImplementedError("Not implemented yet")
 
+    @abstractmethod
     def is_valid(self, point):
         """Is point on-sequence and in-bounds?"""
-        raise NotImplementedError()
+        pass
 
+    @abstractmethod
     def get_prev_point(self, point):
         """Return the previous point < point, or None if out of bounds."""
-        raise NotImplementedError()
+        pass
 
+    @abstractmethod
     def get_nearest_prev_point(self, point):
         """Return the largest point < some arbitrary point."""
-        raise NotImplementedError()
+        pass
 
+    @abstractmethod
     def get_next_point(self, point):
         """Return the next point > point, or None if out of bounds."""
-        raise NotImplementedError()
+        pass
 
+    @abstractmethod
     def get_next_point_on_sequence(self, point):
         """Return the next point > point assuming that point is on-sequence,
         or None if out of bounds."""
-        raise NotImplementedError()
+        pass
 
+    @abstractmethod
     def get_first_point(self, point):
         """Return the first point >= to point, or None if out of bounds."""
-        raise NotImplementedError()
+        pass
 
+    @abstractmethod
     def get_stop_point(self):
         """Return the last point in this sequence, or None if unbounded."""
-        raise NotImplementedError()
+        pass
 
+    @abstractmethod
     def __eq__(self, other):
         # Return True if other (sequence) is equal to self.
-        raise NotImplementedError()
+        pass
+
+
+class TestBaseClasses(unittest.TestCase):
+    """Test the abstract base classes cannot be instantiated on their own
+    """
+    def test_simple_abstract_class_test(self):
+        """Cannot instantiate abstract classes, they must be defined in
+        the subclasses"""
+        self.assertRaises(TypeError, SequenceBase, "sequence-string",
+                          "context_string")
+        self.assertRaises(TypeError, IntervalBase, "value")
+        self.assertRaises(TypeError, PointBase, "value")
+
+
+if __name__ == '__main__':
+    unittest.main()
