@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2016 NIWA
+# Copyright (C) 2008-2017 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@ from isodatetime.dumpers import TimePointDumper
 from isodatetime.parsers import TimePointParser, DurationParser
 from isodatetime.timezone import (
     get_local_time_zone, get_local_time_zone_format)
-from cylc.syntax_flags import set_syntax_version, VERSION_PREV, VERSION_NEW
 from cylc.time_parser import CylcTimeParser
 from cylc.cycling import (
     PointBase, IntervalBase, SequenceBase, PointParsingError,
@@ -38,18 +37,8 @@ CYCLER_TYPE_SORT_KEY_ISO8601 = "b"
 
 MEMOIZE_LIMIT = 10000
 
-OLD_STRPTIME_FORMATS_BY_LENGTH = {
-    4: "%Y",
-    6: "%Y%m",
-    8: "%Y%m%d",
-    10: "%Y%m%d%H",
-    12: "%Y%m%d%H%M",
-    14: "%Y%m%d%H%M%S",
-}
 DATE_TIME_FORMAT = "CCYYMMDDThhmm"
 EXPANDED_DATE_TIME_FORMAT = "+XCCYYMMDDThhmm"
-PREV_DATE_TIME_FORMAT = "%Y%m%d%H"
-PREV_DATE_TIME_REC = re.compile("^\d{10}$")
 NEW_DATE_TIME_REC = re.compile("T")
 
 WARNING_PARSE_EXPANDED_YEAR_DIGITS = (
@@ -309,7 +298,6 @@ class ISO8601Sequence(SequenceBase):
 
     def __init__(self, dep_section, context_start_point=None,
                  context_end_point=None):
-
         self.dep_section = dep_section
 
         if context_start_point is None:
@@ -325,37 +313,42 @@ class ISO8601Sequence(SequenceBase):
 
         self.offset = ISO8601Interval.get_null()
 
-        recurrence_syntax = convert_old_cycler_syntax(
-            dep_section, start_point=self.context_start_point)
-
-        if not recurrence_syntax:
-            raise ValueError(
-                "ERROR: bad cycling sequence syntax: %s" % dep_section)
-
         self._cached_first_point_values = {}
         self._cached_next_point_values = {}
         self._cached_valid_point_booleans = {}
         self._cached_recent_valid_points = []
 
-        self.spec = recurrence_syntax
-        self.custom_point_parse_function = None
-        if SuiteSpecifics.DUMP_FORMAT == PREV_DATE_TIME_FORMAT:
-            self.custom_point_parse_function = point_parse
+        self.spec = dep_section
         self.abbrev_util = CylcTimeParser(
             self.context_start_point, self.context_end_point,
             num_expanded_year_digits=SuiteSpecifics.NUM_EXPANDED_YEAR_DIGITS,
             dump_format=SuiteSpecifics.DUMP_FORMAT,
-            custom_point_parse_function=self.custom_point_parse_function,
             assumed_time_zone=SuiteSpecifics.ASSUMED_TIME_ZONE
         )
-        self.recurrence, excl_point = self.abbrev_util.parse_recurrence(
-            recurrence_syntax)
-        self.exclusion = ISO8601Point.from_nonstandard_string(
-            str(excl_point)) if excl_point else None
+
+        # Parse_recurrence returns an isodatetime TimeRecurrence object
+        # and a list of exclusion strings.
+        self.recurrence, excl_points = self.abbrev_util.parse_recurrence(
+            dep_section)  # should this be self.dep_section??
+
+        # Convert (potentially) non-standard strings to ISO8601Point objects.
+        self.exclusions = []
+        for point in excl_points:
+            exclusion = ISO8601Point.from_nonstandard_string(
+                str(point)) if point else None
+            self.exclusions.append(exclusion)
+
+        # Convert the exclusion ISO8601Points to TimePoints so they can
+        # be compared with the TimeRecurrence object later.
+        self.p_iso_exclusions = []
+        for timepoint in self.exclusions:
+            self.p_iso_exclusions.append(point_parse(str(timepoint)))
+
         self.step = ISO8601Interval(str(self.recurrence.duration))
         self.value = str(self.recurrence)
-        if self.exclusion:
-            self.value += '!' + str(self.exclusion)
+        # Concatenate the strings in exclusion list
+        if self.exclusions:
+            self.value += '!' + str(self.exclusions)
 
     def get_interval(self):
         """Return the interval between points in this sequence."""
@@ -375,14 +368,14 @@ class ISO8601Sequence(SequenceBase):
         self._cached_next_point_values = {}
         self._cached_valid_point_booleans = {}
         self._cached_recent_valid_points = []
-        self.value = str(self.recurrence) + '!' + str(self.exclusion)
-        if self.exclusion:
-            self.value += '!' + str(self.exclusion)
+        self.value = str(self.recurrence) + '!' + str(self.exclusions)
+        if self.exclusions:
+            self.value += '!' + str(self.exclusions)
 
     def is_on_sequence(self, point):
         """Return True if point is on-sequence."""
         # Iterate starting at recent valid points, for speed.
-        if self.exclusion and point == self.exclusion:
+        if self.exclusions and point in self.exclusions:
             return False
         for valid_point in reversed(self._cached_recent_valid_points):
             if valid_point == point:
@@ -421,7 +414,11 @@ class ISO8601Sequence(SequenceBase):
                 raise SequenceDegenerateError(self.recurrence,
                                               SuiteSpecifics.DUMP_FORMAT,
                                               res, point)
-            if self.exclusion and res == self.exclusion:
+            # Check if res point is in the list of exclusions
+            # If so, check the previous point by recursion.
+            # Once you have found a point that is *not* in the exclusion
+            # list, you can return it.
+            if self.exclusions and res in self.exclusions:
                 return self.get_prev_point(res)
         return res
 
@@ -430,14 +427,12 @@ class ISO8601Sequence(SequenceBase):
         if self.is_on_sequence(point):
             return self.get_prev_point(point)
         p_iso_point = point_parse(point.value)
-        p_iso_excl = None
-        if self.exclusion:
-            p_iso_excl = point_parse(self.exclusion.value)
         prev_iso_point = None
+
         for recurrence_iso_point in self.recurrence:
+            # Is recurrence point greater than aribitrary point?
             if (recurrence_iso_point > p_iso_point or
-                    (p_iso_excl and recurrence_iso_point == p_iso_excl)):
-                # Technically, >=, but we already test for this above.
+                    (recurrence_iso_point in self.p_iso_exclusions)):
                 break
             prev_iso_point = recurrence_iso_point
         if prev_iso_point is None:
@@ -448,7 +443,8 @@ class ISO8601Sequence(SequenceBase):
                 self.recurrence, SuiteSpecifics.DUMP_FORMAT,
                 nearest_point, point
             )
-        if self.exclusion and nearest_point == self.exclusion:
+        # Check all exclusions
+        if self.exclusions and nearest_point in self.exclusions:
             return self.get_prev_point(nearest_point)
         return nearest_point
 
@@ -467,7 +463,7 @@ class ISO8601Sequence(SequenceBase):
             while next_point is not None and (next_point <= point or excluded):
                 excluded = False
                 next_point = self.get_next_point_on_sequence(next_point)
-                if next_point and next_point == self.exclusion:
+                if next_point and next_point in self.exclusions:
                     excluded = True
             if next_point is not None:
                 self._check_and_cache_next_point(point, next_point)
@@ -477,7 +473,7 @@ class ISO8601Sequence(SequenceBase):
         for recurrence_iso_point in self.recurrence:
             if recurrence_iso_point > p_iso_point:
                 next_point = ISO8601Point(str(recurrence_iso_point))
-                if next_point and next_point == self.exclusion:
+                if next_point and next_point in self.exclusions:
                     continue
                 self._check_and_cache_next_point(point, next_point)
                 return next_point
@@ -516,7 +512,8 @@ class ISO8601Sequence(SequenceBase):
                     self.recurrence, SuiteSpecifics.DUMP_FORMAT,
                     point, result
                 )
-        if result and result == self.exclusion:
+        # Check it is in the exclusions list now
+        if result and result in self.exclusions:
             return self.get_next_point_on_sequence(result)
         return result
 
@@ -531,7 +528,8 @@ class ISO8601Sequence(SequenceBase):
             if recurrence_iso_point >= p_iso_point:
                 first_point_value = str(recurrence_iso_point)
                 ret = ISO8601Point(first_point_value)
-                if ret and ret == self.exclusion:
+                # Check multiple exclusions
+                if ret and ret in self.exclusions:
                     return self.get_next_point_on_sequence(ret)
                 if (len(self._cached_first_point_values) >
                         self._MAX_CACHED_POINTS):
@@ -545,7 +543,8 @@ class ISO8601Sequence(SequenceBase):
         """Return the first point in this sequence, or None."""
         for recurrence_iso_point in self.recurrence:
             point = ISO8601Point(str(recurrence_iso_point))
-            if not self.exclusion or point != self.exclusion:
+            # Check for multiple exclusions
+            if not self.exclusions or point not in self.exclusions:
                 return point
         return None
 
@@ -562,7 +561,7 @@ class ISO8601Sequence(SequenceBase):
                 prev = curr
                 curr = recurrence_iso_point
             ret = ISO8601Point(str(recurrence_iso_point))
-            if self.exclusion and ret == self.exclusion:
+            if self.exclusions and ret in self.exclusions:
                 return ISO8601Point(str(prev))
             return ret
         return None
@@ -579,34 +578,6 @@ class ISO8601Sequence(SequenceBase):
         return self.value
 
 
-def convert_old_cycler_syntax(dep_section, only_detect_old=False,
-                              start_point=None):
-    """Convert old cycler syntax into our Cylc-ISO8601 format."""
-    for re_old_format, unit in [
-            ("^Daily\(\s*(\d+)\s*,\s*(\d+)\s*\)$", "D"),
-            ("^Monthly\(\s*(\d+)\s*,\s*(\d+)\s*\)$", "M"),
-            ("^Yearly\(\s*(\d+)\s*,\s*(\d+)\s*\)$", "Y")]:
-        results = re.search(re_old_format, dep_section)
-        if not results:
-            continue
-        if only_detect_old:
-            return True
-        anchor, step = results.groups()
-        step = ISO8601Interval("P%s%s" % (step, unit))
-        return _get_old_anchor_step_recurrence(anchor, step, start_point)
-    # Check for the hourly syntax.
-    results = re.match('(0?[0-9]|1[0-9]|2[0-3])$', dep_section)
-    if results:
-        # back compat 0,6,12 etc.
-        if only_detect_old:
-            return True
-        anchor = results.groups()[0]
-        return "T%02d/PT24H" % int(anchor)
-    if only_detect_old:
-        return False
-    return dep_section
-
-
 def _get_old_anchor_step_recurrence(anchor, step, start_point):
     """Return a string representing an old-format recurrence translation."""
     anchor_point = ISO8601Point.from_nonstandard_string(anchor)
@@ -628,70 +599,6 @@ def init_from_cfg(cfg):
     assume_utc = cfg['cylc']['UTC mode']
     cycling_mode = cfg['scheduling']['cycling mode']
 
-    # Detect (date-time) previous-format cycle point usage.
-    has_prev_format_cycle_point = (
-        (initial_cycle_point is not None and
-         PREV_DATE_TIME_REC.search(initial_cycle_point)) or
-        (final_cycle_point is not None and
-         PREV_DATE_TIME_REC.search(final_cycle_point))
-    )
-    if (has_prev_format_cycle_point and
-            custom_dump_format != PREV_DATE_TIME_FORMAT):
-        set_syntax_version(
-            VERSION_PREV,
-            "initial/final cycle point format: CCYYMMDDhh"
-        )
-
-    # Detect (date-time) ISO 8601-format cycle point usage.
-    has_new_format_cycle_point = (
-        (initial_cycle_point is not None and
-         NEW_DATE_TIME_REC.search(initial_cycle_point)) or
-        (final_cycle_point is not None and
-         NEW_DATE_TIME_REC.search(final_cycle_point))
-    )
-    if has_new_format_cycle_point:
-        set_syntax_version(
-            VERSION_NEW,
-            "initial/final cycle point format: non-numeric (ISO 8601?)"
-        )
-
-    # Loop over all dependency sub-sections and detect cycler syntax versions.
-    dep_sections = list(cfg['scheduling']['dependencies'])
-    while dep_sections:
-        dep_section = dep_sections.pop(0)
-        if re.search("(?![^(]+\)),", dep_section):
-            dep_sections.extend(
-                [i.strip() for i in re.split("(?![^(]+\)),", dep_section)])
-            continue
-        if dep_section == "graph":
-            if cfg['scheduling']['dependencies']['graph']:
-                # Using async graph in date-time cycling.
-                set_syntax_version(
-                    VERSION_PREV,
-                    "[scheduling][[dependencies]]graph: mixed with " +
-                    "date-time cycling"
-                )
-                custom_dump_format = PREV_DATE_TIME_FORMAT
-                num_expanded_year_digits = 0
-            continue
-        if convert_old_cycler_syntax(dep_section,
-                                     only_detect_old=True):
-            # Detected prev-format (old) syntax.
-            set_syntax_version(
-                VERSION_PREV,
-                "[scheduling][[dependencies]][[[%s]]]: old-style cycling" %
-                dep_section
-            )
-            custom_dump_format = PREV_DATE_TIME_FORMAT
-            num_expanded_year_digits = 0
-        else:
-            # Detected new-style syntax.
-            set_syntax_version(
-                VERSION_NEW,
-                ("[scheduling][[dependencies]][[[%s]]]: " % dep_section) +
-                "ISO 8601-style cycling"
-            )
-
     init(
         num_expanded_year_digits=num_expanded_year_digits,
         custom_dump_format=custom_dump_format,
@@ -741,14 +648,10 @@ def init(num_expanded_year_digits=0, custom_dump_format=None, time_zone=None,
         dump_format=SuiteSpecifics.DUMP_FORMAT,
         assumed_time_zone=time_zone_hours_minutes
     )
-    custom_point_parse_function = None
-    if SuiteSpecifics.DUMP_FORMAT == PREV_DATE_TIME_FORMAT:
-        custom_point_parse_function = point_parse
     SuiteSpecifics.abbrev_util = CylcTimeParser(
         None, None,
         num_expanded_year_digits=SuiteSpecifics.NUM_EXPANDED_YEAR_DIGITS,
         dump_format=SuiteSpecifics.DUMP_FORMAT,
-        custom_point_parse_function=custom_point_parse_function,
         assumed_time_zone=SuiteSpecifics.ASSUMED_TIME_ZONE
     )
 
@@ -781,6 +684,16 @@ def interval_parse(interval_string):
                 interval_string.replace("+", "", 1)).copy()
 
 
+def is_offset_absolute(offset_string):
+    """Return True if offset_string is a point rather than an interval."""
+    try:
+        interval_parse(offset_string)
+    except Exception:
+        return True
+    else:
+        return False
+
+
 @memoize
 def _interval_parse(interval_string):
     """Parse an interval_string into a proper Duration object."""
@@ -796,39 +709,14 @@ def point_parse(point_string):
 def _point_parse(point_string):
     """Parse a point_string into a proper TimePoint object."""
     if "%" in SuiteSpecifics.DUMP_FORMAT:
-        # Includes prev-format point strings.
+        # May be a custom not-quite ISO 8601 dump format.
         try:
-            point = SuiteSpecifics.point_parser.strptime(
+            return SuiteSpecifics.point_parser.strptime(
                 point_string, SuiteSpecifics.DUMP_FORMAT)
-        except ValueError as e:
-            strptime_string = _get_old_strptime_format(point_string)
-            if strptime_string is not None:
-                return SuiteSpecifics.point_parser.strptime(
-                    point_string, strptime_string)
-        else:
-            return point
-    # Attempt to parse it in ISO 8601 format then...
-    try:
-        point = SuiteSpecifics.point_parser.parse(point_string)  # Fail?
-        return point
-    except ValueError:
-        strptime_string = _get_old_strptime_format(point_string)
-        if strptime_string is None:
-            raise
-        set_syntax_version(
-            VERSION_PREV,
-            "non-ISO-8601-compatible cycle point: %s" % point_string
-        )
-        return SuiteSpecifics.point_parser.strptime(
-            point_string, strptime_string)
-
-
-def _get_old_strptime_format(point_string):
-    """Return an adjusted strptime format depending on the string length."""
-    try:
-        return OLD_STRPTIME_FORMATS_BY_LENGTH[len(point_string)]
-    except KeyError:
-        return None
+        except ValueError:
+            pass
+    # Attempt to parse it in ISO 8601 format.
+    return SuiteSpecifics.point_parser.parse(point_string)
 
 
 class TestISO8601Sequence(unittest.TestCase):
@@ -850,6 +738,70 @@ class TestISO8601Sequence(unittest.TestCase):
         self.assertEqual(output, ['20000101T0000Z', '20000101T0100Z',
                                   '20000101T0300Z', '20000101T0400Z'])
 
+    def test_multiple_exclusions_complex1(self):
+        """Tests sequences that have multiple exclusions and a more
+        complicated format"""
+
+        # A sequence that specifies a dep start time
+        sequence = ISO8601Sequence('20000101T01Z/PT1H!20000101T02Z',
+                                   '20000101T01Z')
+
+        output = []
+        point = sequence.get_start_point()
+        count = 0
+        # We are going to make four sequence points
+        while point and count < 4:
+            output.append(point)
+            point = sequence.get_next_point(point)
+            count += 1
+        output = [str(out) for out in output]
+        # We should expect one of the hours to be excluded: T02
+        self.assertEqual(output, ['20000101T0100Z', '20000101T0300Z',
+                                  '20000101T0400Z', '20000101T0500Z'])
+
+    def test_multiple_exclusions_complex2(self):
+        """Tests sequences that have multiple exclusions and a more
+        complicated format"""
+
+        # A sequence that specifies a dep start time
+        sequence = ISO8601Sequence('20000101T01Z/PT1H!'
+                                   '(20000101T02Z,20000101T03Z)',
+                                   '20000101T00Z',
+                                   '20000101T05Z')
+
+        output = []
+        point = sequence.get_start_point()
+        count = 0
+        # We are going to make four sequence points
+        while point and count < 3:
+            output.append(point)
+            point = sequence.get_next_point(point)
+            count += 1
+        output = [str(out) for out in output]
+        # We should expect two of the hours to be excluded: T02, T03
+        self.assertEqual(output, ['20000101T0100Z', '20000101T0400Z',
+                                  '20000101T0500Z'])
+
+    def test_multiple_exclusions_simple(self):
+        """Tests generation of points for sequences with multiple exclusions
+        """
+        init(time_zone='Z')
+        sequence = ISO8601Sequence('PT1H!(20000101T02Z,20000101T03Z)',
+                                   '20000101T00Z')
+
+        output = []
+        point = sequence.get_start_point()
+        count = 0
+        # We are going to make four sequence points
+        while point and count < 4:
+            output.append(point)
+            point = sequence.get_next_point(point)
+            count += 1
+        output = [str(out) for out in output]
+        # We should expect two of the hours to be excluded: T02 and T03
+        self.assertEqual(output, ['20000101T0000Z', '20000101T0100Z',
+                                  '20000101T0400Z', '20000101T0500Z'])
+
     def test_exclusions_extensive(self):
         """Test ISO8601Sequence methods for sequences with exclusions"""
         init(time_zone='+05')
@@ -874,6 +826,49 @@ class TestISO8601Sequence(unittest.TestCase):
         self.assertEqual(sequence.get_first_point(point_0), point_1)
         self.assertEqual(sequence.get_start_point(), point_1)
 
+    def test_multiple_exclusions_extensive(self):
+        """Test ISO8601Sequence methods for sequences with multiple exclusions
+        """
+        init(time_zone='+05')
+        sequence = ISO8601Sequence('PT1H!(20000101T02Z,20000101T03Z)',
+                                   '20000101T00Z',
+                                   '20000101T06Z')
+
+        point_0 = ISO8601Point('20000101T00Z')
+        point_1 = ISO8601Point('20000101T01Z')
+        point_2 = ISO8601Point('20000101T02Z')  # First excluded point
+        point_3 = ISO8601Point('20000101T03Z')  # Second excluded point
+        point_4 = ISO8601Point('20000101T04Z')
+
+        # Check the excluded points are not on the sequence
+        self.assertFalse(sequence.is_on_sequence(point_2))
+        self.assertFalse(sequence.is_on_sequence(point_3))
+        self.assertFalse(sequence.is_valid(point_2))  # Should be excluded
+        self.assertFalse(sequence.is_valid(point_3))  # Should be excluded
+        # Check that we can correctly retrieve previous points
+        self.assertEqual(sequence.get_prev_point(point_2), point_1)
+        # Should skip two excluded points
+        self.assertEqual(sequence.get_prev_point(point_4), point_1)
+        self.assertEqual(sequence.get_nearest_prev_point(point_2), point_1)
+        self.assertEqual(sequence.get_nearest_prev_point(point_4), point_1)
+        self.assertEqual(sequence.get_next_point(point_1), point_4)
+        self.assertEqual(sequence.get_next_point(point_3), point_4)
+
+        sequence = ISO8601Sequence('PT1H!20000101T00Z', '20000101T00Z')
+        # Check that the first point is after 00.
+        self.assertEqual(sequence.get_first_point(point_0), point_1)
+        self.assertEqual(sequence.get_start_point(), point_1)
+
+        # Check a longer list of exclusions
+        # Also note you can change the format of the exclusion list
+        # (removing the parentheses)
+        sequence = ISO8601Sequence('PT1H! 20000101T02Z, 20000101T03Z,'
+                                   '20000101T04Z',
+                                   '20000101T00Z',
+                                   '20000101T06Z')
+        self.assertEqual(sequence.get_nearest_prev_point(point_3), point_1)
+        self.assertEqual(sequence.get_nearest_prev_point(point_4), point_1)
+
     def test_simple(self):
         """Run some simple tests for date-time cycling."""
         init(time_zone='Z')
