@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2016 NIWA
+# Copyright (C) 2008-2017 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 
 import time
 from cylc.task_id import TaskID
-from cylc.network.suite_state import SUITE_STATUS_STOPPED
+from cylc.network.suite_state_client import SUITE_STATUS_STOPPED
 from cylc.task_state import TASK_STATUS_READY
 
 
