@@ -1,7 +1,7 @@
-#!usr/bin/env python
+#!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2016 NIWA
+# Copyright (C) 2008-2017 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -102,7 +102,7 @@ class GraphParser(object):
         (
         (?:(?:!)?\w[\w\-+%@]*)  # node name
         (?:<[\w,=\-+]+>)?       # optional parameter list
-        (?:\[[\w\-\+\^]+\])?    # optional cycle point offset
+        (?:\[[\w\-\+\^:]+\])?   # optional cycle point offset
         (?::[\w\-]+)?           # optional trigger type
         )
     ''', re.X)           # end of string
@@ -110,7 +110,7 @@ class GraphParser(object):
     # Extract node info from a left-side expression, after parameter expansion.
     REC_NODES = re.compile(r'''
         ((?:!)?\w[\w\-+%@]*)  # node name
-        (\[[\w\-\+\^]+\])?    # optional cycle point offset
+        (\[[\w\-\+\^:]+\])?   # optional cycle point offset
         (:[\w\-]+)?           # optional trigger type
     ''', re.X)
 
@@ -125,9 +125,9 @@ class GraphParser(object):
     REC_PARAMETERS = re.compile(r'<[\w,=\-+]+>')
 
     # Detect and extract suite state polling task info.
-    REC_SUITE_STATE = re.compile('(\w+)(<([\w\.\-]+)::(\w+)(:\w+)?>)')
+    REC_SUITE_STATE = re.compile('(\w+)(<([\w\.\-/]+)::(\w+)(:\w+)?>)')
 
-    def __init__(self, family_map=None, parameters=None, startup_tasks=[]):
+    def __init__(self, family_map=None, parameters=None):
         """Initializing the graph string parser.
 
         family_map (empty or None if no families) is:
@@ -140,13 +140,9 @@ class GraphParser(object):
         self.triggers = {}
         self.original = {}
         self.suite_state_polling_tasks = {}
-        # Back-compat cylc-5:
-        self.startup_tasks = startup_tasks
-        self.get_startup = False
         self.r1_point = None
-        self.startup_graph_text = ''
 
-    def parse_graph(self, graph_string, get_startup=False, r1_point=None):
+    def parse_graph(self, graph_string, r1_point=None):
         """Parse the graph string for a single graph section.
 
         (Assumes any general line-continuation markers have been processed).
@@ -159,7 +155,6 @@ class GraphParser(object):
               i. Replace families with members (any or all semantics).
              ii. Record parsed dependency information for each right-side node.
         """
-        self.get_startup = get_startup
         self.r1_point = r1_point
 
         # Strip comments, whitespace, and blank lines.
@@ -260,7 +255,7 @@ class GraphParser(object):
             # Auto-trigger lone nodes and initial nodes in a chain.
             for name, offset, _ in self.__class__.REC_NODES.findall(chain[0]):
                 if not offset:
-                    pairs.add(('', name))
+                    pairs.add((None, name))
             for i in range(0, len(chain) - 1):
                 pairs.add((chain[i], chain[i + 1]))
 
@@ -280,42 +275,61 @@ class GraphParser(object):
         Trigger qualifiers, but not cycle offsets, are ignored on the right to
         allow chaining.
         """
-        if self.__class__.OP_OR in right:
+        # Raise error for right-hand-side OR operators.
+        if right and self.__class__.OP_OR in right:
             raise GraphParseError("ERROR, illegal OR on RHS: %s" % right)
-        # Remove aualifiers from right-side nodes.
-        for qual in self.__class__.REC_TRIG_QUAL.findall(right):
-            right = right.replace(qual, '')
-        if self.__class__.SUICIDE_MARK in left:
+
+        # Remove qualifiers from right-side nodes.
+        if right:
+            for qual in self.__class__.REC_TRIG_QUAL.findall(right):
+                right = right.replace(qual, '')
+
+        # Raise error if suicide triggers on the left of the trigger.
+        if left and self.__class__.SUICIDE_MARK in left:
             raise GraphParseError(
                 "ERROR, suicide markers must be"
                 " on the right of a trigger: %s" % left)
+
         # Cycle point offsets are not allowed on the right side (yet).
-        if '[' in right:
+        if right and '[' in right:
             raise GraphParseError(
                 "ERROR, illegal cycle point offset on the right: %s => %s" % (
                     left, right))
+
         # Check that parentheses match.
-        if left.count("(") != left.count(")"):
+        if left and left.count("(") != left.count(")"):
             raise GraphParseError(
                 "ERROR, parenthesis mismatch in: \"" + left + "\"")
 
         # Split right side on AND.
         rights = right.split(self.__class__.OP_AND)
+        if '' in rights or right and not all(rights):
+            raise GraphParseError(
+                "ERROR, null task name in graph: %s=>%s" % (left, right))
 
-        if self.__class__.OP_OR in left or '(' in left:
+        if not left or (self.__class__.OP_OR in left or '(' in left):
             # Treat conditional or bracketed expressions as a single entity.
             lefts = [left]
         else:
             # Split non-conditional left-side expressions on AND.
             lefts = left.split(self.__class__.OP_AND)
+        if '' in lefts or left and not all(lefts):
+            raise GraphParseError(
+                "ERROR, null task name in graph: %s=>%s" % (left, right))
 
         for left in lefts:
-            # Extract infomation about all nodes on the left.
-            info = self.__class__.REC_NODES.findall(left)
+            # Extract information about all nodes on the left.
+
+            if left:
+                info = self.__class__.REC_NODES.findall(left)
+                expr = left
+            else:
+                # There is no left-hand-side task.
+                info = []
+                expr = ''
 
             # Make success triggers explicit.
             n_info = []
-            expr = left
             for name, offset, trig in info:
                 if not trig:
                     trig = self.__class__.TRIG_SUCCEED
@@ -330,7 +344,7 @@ class GraphParser(object):
             info = n_info
 
             # Determine semantics of all family triggers present.
-            families = {}
+            family_trig_map = {}
             for name, offset, trig in info:
                 if name in self.family_map:
                     if trig.endswith(self.__class__.FAM_TRIG_EXT_ANY):
@@ -343,15 +357,15 @@ class GraphParser(object):
                         # Unqualified (FAM => foo) or bad (FAM:bad => foo).
                         raise GraphParseError(
                             "ERROR, bad family trigger in %s" % expr)
-                    families[name] = (ttype, ext)
+                    family_trig_map[(name, trig)] = (ttype, ext)
                 else:
                     if (trig.endswith(self.__class__.FAM_TRIG_EXT_ANY) or
                             trig.endswith(self.__class__.FAM_TRIG_EXT_ALL)):
                         raise GraphParseError("ERROR, family trigger on non-"
                                               "family namespace %s" % expr)
-            self._families_all_to_all(expr, rights, info, families)
+            self._families_all_to_all(expr, rights, info, family_trig_map)
 
-    def _families_all_to_all(self, expr, rights, info, families):
+    def _families_all_to_all(self, expr, rights, info, family_trig_map):
         """Replace all family names with member names, for all/any semantics.
 
         (Also for graph segments with no family names.)
@@ -359,8 +373,8 @@ class GraphParser(object):
         n_info = []
         n_expr = expr
         for name, offset, trig in info:
-            if name in families:
-                ttype, extn = families[name]
+            if (name, trig) in family_trig_map:
+                ttype, extn = family_trig_map[(name, trig)]
                 m_info = []
                 m_expr = []
                 for mem in self.family_map[name]:
@@ -395,15 +409,8 @@ class GraphParser(object):
         """Store trigger info from "expr => right" for a single task ('right').
 
         Arg info is [(name, offset, trigger_type)] for each name in expr.
-        """
-        if self.startup_tasks:
-            self._add_trigger_back_compat(
-                orig_expr, right, expr, info, suicide)
-        else:
-            self._add_trigger_new(orig_expr, right, expr, info, suicide)
 
-    def _add_trigger_new(self, orig_expr, right, expr, info, suicide):
-        """New __add_trigger() for cylc-7+ or if no cylc-5 start-up tasks."""
+        """
         trigs = []
         for name, offset, trigger in info:
             # Replace finish triggers (must be done after member substn).
@@ -428,66 +435,6 @@ class GraphParser(object):
         self.triggers[right][expr] = (trigs, suicide)
         self.original[right][expr] = orig_expr
 
-    def _add_trigger_back_compat(self, orig_expr, right, expr, info, suicide):
-        """Temporary (pre cylc-7) __add_trigger() for cylc-5 start-up tasks.
-
-        Extract all start-up triggers for use in new R1 sections.
-        """
-        if right in self.startup_tasks:
-            if not self.get_startup or self.r1_point:
-                return
-            elif self.r1_point and suicide:
-                return
-        # Replace finish triggers here (must be done after member substn).
-        trigs = []
-        pruned = []
-        startup_found = False
-        for name, offset, trigger in info:
-            if name in self.startup_tasks:
-                startup_found = True
-            if (self.get_startup and name in self.startup_tasks and
-                    self.r1_point and not offset):
-                that_offset = '[^]'
-            else:
-                that_offset = offset
-            if ((not self.get_startup and name in self.startup_tasks) or
-                    (self.get_startup and name not in self.startup_tasks)):
-                pruned.append("%s%s%s" % (name, that_offset, trigger))
-            elif trigger == self.__class__.TRIG_FINISH:
-                this = "%s%s%s" % (name, re.escape(offset), trigger)
-                that = "(%s%s%s%s%s%s%s)" % (
-                    name, that_offset, self.__class__.TRIG_SUCCEED,
-                    self.__class__.OP_OR,
-                    name, that_offset, self.__class__.TRIG_FAIL)
-                expr = re.sub(this, that, expr)
-                trigs += [
-                    "%s%s%s" % (name,
-                                that_offset, self.__class__.TRIG_SUCCEED),
-                    "%s%s%s" % (name, that_offset, self.__class__.TRIG_FAIL)]
-            else:
-                this = "%s%s%s" % (name, re.escape(offset), trigger)
-                that = "%s%s%s" % (name, that_offset, trigger)
-                expr = re.sub(this, that, expr)
-                trigs += ["%s%s%s" % (name, that_offset, trigger)]
-        if (self.get_startup and not startup_found and
-                right not in self.startup_tasks):
-            return
-        for node in pruned:
-            expr = re.sub(re.escape(node) + '&', '', expr)
-            expr = re.sub('&' + re.escape(node), '', expr)
-            # TODO - IS THIS NEEDED?:
-            expr = re.sub(re.escape(node), '', expr)
-        if self.get_startup:
-            if expr and right:
-                self.startup_graph_text += '\n%s => %s' % (expr, right)
-            elif right:
-                self.startup_graph_text += '\n%s' % right
-        if right not in self.triggers:
-            self.triggers[right] = {}
-            self.original[right] = {}
-        self.triggers[right][expr] = (trigs, suicide)
-        self.original[right][expr] = orig_expr
-
     def _tidy_triggers(self):
         """Remove any null triggers for tasks that also have non-null triggers.
 
