@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2016 NIWA
+# Copyright (C) 2008-2017 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,28 +19,29 @@
 
 import re
 import sys
-import Pyro
 import atexit
 import gobject
 import threading
 from time import sleep, time
 import traceback
 
-from cylc.exceptions import PortFileError
 import cylc.flags
 from cylc.dump import get_stop_state_summary
 from cylc.gui.cat_state import cat_state
-from cylc.network.suite_state import (
-    StateSummaryClient, SuiteStillInitialisingError, get_suite_status_string,
+from cylc.network import ConnectionError, ConnectionDeniedError
+from cylc.network.suite_state_client import (
+    StateSummaryClient, get_suite_status_string,
     SUITE_STATUS_NOT_CONNECTED, SUITE_STATUS_CONNECTED,
-    SUITE_STATUS_INITIALISING, SUITE_STATUS_STOPPED, SUITE_STATUS_STOPPING)
-from cylc.network.suite_info import SuiteInfoClient
-from cylc.network.suite_log import SuiteLogClient
-from cylc.network.suite_command import SuiteCommandClient
+    SUITE_STATUS_INITIALISING, SUITE_STATUS_STOPPED, SUITE_STATUS_STOPPING
+)
+from cylc.network.suite_info_client import SuiteInfoClient
+from cylc.network.suite_log_client import SuiteLogClient
+from cylc.network.suite_command_client import SuiteCommandClient
 from cylc.wallclock import (
     get_current_time_string,
     get_seconds_as_interval_string,
-    get_time_string_from_unix_time)
+    get_time_string_from_unix_time
+)
 from cylc.task_id import TaskID
 from cylc.version import CYLC_VERSION
 from cylc.gui.warning_dialog import warning_dialog
@@ -135,7 +136,7 @@ class Updater(threading.Thread):
         self.cfg = app.cfg
         self.info_bar = app.info_bar
 
-        self._summary_update_time = None
+        self.summary_update_time = None
         self.err_log_lines = []
         self._err_num_log_lines = 10
         self.err_log_size = 0
@@ -174,8 +175,7 @@ class Updater(threading.Thread):
         self.version_mismatch_warned = False
 
         client_args = (self.cfg.suite, self.cfg.owner, self.cfg.host,
-                       self.cfg.pyro_timeout, self.cfg.port, self.cfg.db,
-                       self.cfg.my_uuid)
+                       self.cfg.port, self.cfg.comms_timeout, self.cfg.my_uuid)
         self.state_summary_client = StateSummaryClient(*client_args)
         self.suite_info_client = SuiteInfoClient(*client_args)
         self.suite_log_client = SuiteLogClient(*client_args)
@@ -187,7 +187,7 @@ class Updater(threading.Thread):
         """Try to reconnect to the suite daemon."""
         if cylc.flags.debug:
             print >> sys.stderr, "  reconnection...",
-        # Reset Pyro clients.
+        # Reset comms clients.
         self.suite_log_client.reset()
         self.state_summary_client.reset()
         self.suite_info_client.reset()
@@ -195,37 +195,45 @@ class Updater(threading.Thread):
         try:
             self.daemon_version = self.suite_info_client.get_info(
                 'get_cylc_version')
-        except KeyError:
-            self.daemon_version = "??? (pre 6.1.2?)"
+        except ConnectionDeniedError as exc:
             if cylc.flags.debug:
-                print >> sys.stderr, "succeeded (old daemon)"
-        except PortFileError as exc:
+                traceback.print_exc()
+            if not self.connect_fail_warned:
+                self.connect_fail_warned = True
+                gobject.idle_add(
+                    self.warn,
+                    "ERROR: %s\n\nIncorrect suite passphrase?" % exc)
+            return
+        except ConnectionError as exc:
+            # Failed to (re)connect
+            # Suite not running, starting up or just stopped.
             if cylc.flags.debug:
                 traceback.print_exc()
             # Use info bar to display stop summary if available.
             # Otherwise, just display the reconnect count down.
             if self.cfg.suite and self.stop_summary is None:
-                self.stop_summary = get_stop_state_summary(
+                stop_summary = get_stop_state_summary(
                     cat_state(self.cfg.suite, self.cfg.host, self.cfg.owner))
                 self.last_update_time = time()
-            if self.stop_summary is not None and any(self.stop_summary):
-                gobject.idle_add(
-                    self.info_bar.set_stop_summary, self.stop_summary)
-            else:
-                self.info_bar.set_update_time(
-                    None, self.info_bar.DISCONNECTED_TEXT)
-            return
-        except Pyro.errors.ConnectionDeniedError as exc:
-            if cylc.flags.debug:
-                traceback.print_exc()
-            if not self.connect_fail_warned:
-                gobject.idle_add(
-                    self.warn,
-                    "ERROR: %s\n\nIncorrect suite passphrase?" % exc)
+                if stop_summary != self.stop_summary:
+                    self.stop_summary = stop_summary
+                    self.status = SUITE_STATUS_STOPPED
+                    gobject.idle_add(
+                        self.info_bar.set_stop_summary, stop_summary)
+            try:
+                update_time_str = get_time_string_from_unix_time(
+                    self.stop_summary[0]["last_updated"])
+            except (AttributeError, IndexError, KeyError, TypeError):
+                update_time_str = None
+            gobject.idle_add(
+                self.info_bar.set_update_time,
+                update_time_str, self.info_bar.DISCONNECTED_TEXT)
             return
         except Exception as exc:
             if cylc.flags.debug:
                 traceback.print_exc()
+            if not self.connect_fail_warned:
+                self.connect_fail_warned = True
                 gobject.idle_add(self.warn, str(exc))
             return
 
@@ -270,15 +278,10 @@ class Updater(threading.Thread):
 
     def retrieve_err_log(self):
         """Retrieve suite err log; return True if it has changed."""
-        try:
-            new_err_content, new_err_size = (
-                self.suite_log_client.get_err_content(
-                    self.err_log_size, self._err_num_log_lines))
-        except AttributeError:
-            # TODO: post-backwards compatibility concerns, remove this handling
-            new_err_content = ""
-            new_err_size = self.err_log_size
-
+        new_err_content, new_err_size = (
+            self.suite_log_client.get_err_content(
+                self.err_log_size, self._err_num_log_lines)
+        )
         err_log_changed = (new_err_size != self.err_log_size)
         if err_log_changed:
             self.err_log_lines += new_err_content.splitlines()
@@ -288,48 +291,37 @@ class Updater(threading.Thread):
 
     def retrieve_summary_update_time(self):
         """Retrieve suite summary update time; return True if changed."""
-        do_update = False
-        try:
-            summary_update_time = (
-                self.state_summary_client.get_suite_state_summary_update_time()
-            )
-            if (summary_update_time is None or
-                    self._summary_update_time is None or
-                    summary_update_time != self._summary_update_time):
-                self._summary_update_time = summary_update_time
-                do_update = True
-        except AttributeError:
-            # TODO: post-backwards compatibility concerns, remove this handling
-            # Force an update for daemons using the old API
-            do_update = True
-        return do_update
+        prev_summary_update_time = self.summary_update_time
+        self.summary_update_time = (
+            self.state_summary_client.get_suite_state_summary_update_time())
+        if self.summary_update_time is None:
+            self.set_status(SUITE_STATUS_INITIALISING)
+        else:
+            self.summary_update_time = float(self.summary_update_time)
+        return prev_summary_update_time != self.summary_update_time
 
     def retrieve_state_summaries(self):
         """Retrieve suite summary."""
         glbl, states, fam_states = (
             self.state_summary_client.get_suite_state_summary())
-        self.ancestors = self.suite_info_client.get_info(
-            'get_first_parent_ancestors')
-        self.ancestors_pruned = self.suite_info_client.get_info(
-            'get_first_parent_ancestors', True)
-        self.descendants = self.suite_info_client.get_info(
-            'get_first_parent_descendants')
-        self.all_families = self.suite_info_client.get_info('get_all_families')
+
+        (self.ancestors, self.ancestors_pruned, self.descendants,
+            self.all_families) = self.suite_info_client.get_info(
+                {'function': 'get_first_parent_ancestors'},
+                {'function': 'get_first_parent_ancestors', 'pruned': True},
+                {'function': 'get_first_parent_descendants'},
+                {'function': 'get_all_families'})
 
         self.mode = glbl['run_mode']
 
-        if self.cfg.use_defn_order and 'namespace definition order' in glbl:
-            # (protect for compat with old suite daemons)
+        if self.cfg.use_defn_order:
             nsdo = glbl['namespace definition order']
             if self.ns_defn_order != nsdo:
                 self.ns_defn_order = nsdo
                 self.dict_ns_defn_order = dict(zip(nsdo, range(0, len(nsdo))))
-        try:
-            self.update_time_str = get_time_string_from_unix_time(
-                glbl['last_updated'])
-        except (TypeError, ValueError):
-            # Older suite...
-            self.update_time_str = glbl['last_updated'].isoformat()
+
+        self.update_time_str = get_time_string_from_unix_time(
+            glbl['last_updated'])
         self.global_summary = glbl
 
         if self.restricted_display:
@@ -339,26 +331,15 @@ class Updater(threading.Thread):
         self.full_fam_state_summary = fam_states
         self.refilter()
 
-        try:
-            self.status = glbl['status_string']
-        except KeyError:
-            # Back compat for suite daemons <= 6.9.1.
-            self.status = get_suite_status_string(
-                glbl['paused'], glbl['stopping'], glbl['will_pause_at'],
-                glbl['will_stop_at'])
-
-        try:
-            self.is_reloading = glbl['reloading']
-        except KeyError:
-            # Back compat.
-            pass
+        self.status = glbl['status_string']
+        self.is_reloading = glbl['reloading']
 
     def set_stopped(self):
         """Reset data and clients when suite is stopped."""
         self.connected = False
         self.set_status(SUITE_STATUS_STOPPED)
         self.connect_schd.start()
-        self._summary_update_time = None
+        self.summary_update_time = None
         self.state_summary = {}
         self.full_state_summary = {}
         self.fam_state_summary = {}
@@ -382,9 +363,11 @@ class Updater(threading.Thread):
 
     def set_status(self, status=None):
         """Update status bar."""
+        if status == self.status:
+            return
         if status is not None:
             self.status = status
-        self.info_bar.set_status(self.status)
+        gobject.idle_add(self.info_bar.set_status, self.status)
 
     def warn(self, msg):
         """Pop up a warning dialog; call on idle_add!"""
@@ -416,41 +399,8 @@ class Updater(threading.Thread):
         try:
             err_log_changed = self.retrieve_err_log()
             summaries_changed = self.retrieve_summary_update_time()
-            if summaries_changed:
+            if self.summary_update_time is not None and summaries_changed:
                 self.retrieve_state_summaries()
-        except SuiteStillInitialisingError:
-            # Connection achieved but state summary data not available yet.
-            if cylc.flags.debug:
-                print >> sys.stderr, "  connected, suite initializing ..."
-            self.set_status(SUITE_STATUS_INITIALISING)
-            if self.info_bar.prog_bar_can_start():
-                gobject.idle_add(
-                    self.info_bar.prog_bar_start, SUITE_STATUS_INITIALISING)
-                self.info_bar.set_state([])
-            return False
-        except Pyro.errors.NamingError as exc:
-            if self.daemon_version is not None:
-                # Back compat <= 6.4.0 the state summary object was not
-                # connected to Pyro until initialisation was completed.
-                if cylc.flags.debug:
-                    print >> sys.stderr, (
-                        "  daemon <= 6.4.0, suite initializing ...")
-                self.set_status(SUITE_STATUS_INITIALISING)
-                if self.info_bar.prog_bar_can_start():
-                    gobject.idle_add(self.info_bar.prog_bar_start,
-                                     SUITE_STATUS_INITIALISING)
-                    self.info_bar.set_state([])
-                # Reconnect till we get the suite state object.
-                self.reconnect()
-                return False
-            else:
-                if cylc.flags.debug:
-                    print >> sys.stderr, "  CONNECTION LOST", str(exc)
-                self.set_stopped()
-                if self.info_bar.prog_bar_active():
-                    gobject.idle_add(self.info_bar.prog_bar_stop)
-                self.reconnect()
-                return False
         except Exception as exc:
             if self.status == SUITE_STATUS_STOPPING:
                 # Expected stop: prevent the reconnection warning dialog.
@@ -465,23 +415,17 @@ class Updater(threading.Thread):
         else:
             # Got suite data.
             self.version_mismatch_warned = False
-            if (self.status == SUITE_STATUS_STOPPING and
-                    self.info_bar.prog_bar_can_start()):
-                gobject.idle_add(
-                    self.info_bar.prog_bar_start, self.status)
-            if (self.is_reloading and
-                    self.info_bar.prog_bar_can_start()):
-                gobject.idle_add(
-                    self.info_bar.prog_bar_start, "reloading")
-            if (self.info_bar.prog_bar_active() and
-                    not self.is_reloading and
-                    self.status not in [SUITE_STATUS_STOPPING,
-                                        SUITE_STATUS_INITIALISING]):
+            status_str = None
+            if self.status in [SUITE_STATUS_INITIALISING,
+                               SUITE_STATUS_STOPPING]:
+                status_str = self.status
+            elif self.is_reloading:
+                status_str = "reloading"
+            if status_str is None:
                 gobject.idle_add(self.info_bar.prog_bar_stop)
-            if summaries_changed or err_log_changed:
-                return True
-            else:
-                return False
+            elif self.info_bar.prog_bar_can_start():
+                gobject.idle_add(self.info_bar.prog_bar_start, status_str)
+            return summaries_changed or err_log_changed
 
     def filter_by_name(self, states):
         """Filter by name string."""
@@ -547,8 +491,7 @@ class Updater(threading.Thread):
         self.info_bar.set_mode(self.mode)
         self.info_bar.set_update_time(self.update_time_str)
         self.info_bar.set_status(self.status)
-        self.info_bar.set_log("\n".join(self.err_log_lines),
-                              self.err_log_size)
+        self.info_bar.set_log("\n".join(self.err_log_lines), self.err_log_size)
         return False
 
     def stop(self):
