@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2016 NIWA
+# Copyright (C) 2008-2017 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ from time import sleep
 
 from cylc.task_id import TaskID
 from cylc.gui.dot_maker import DotMaker
-from cylc.network.suite_state import get_id_summary
+from cylc.network.suite_state_client import get_id_summary
 from cylc.wallclock import get_time_string_from_unix_time
 from cylc.task_state import TASK_STATUSES_AUTO_EXPAND
 
@@ -232,10 +232,10 @@ class TreeUpdater(threading.Thread):
                     # Family timing currently left empty.
                     for dt in tkeys:
                         t_info[dt] = ""
-                        t_info['mean_total_elapsed_time_string'] = ""
+                        t_info['mean_elapsed_time_string'] = ""
                         t_info['progress'] = 0
                 else:
-                    meant = summary[id].get('mean total elapsed time')
+                    meant = summary[id].get('mean_elapsed_time')
                     tstart = summary[id].get('started_time')
                     tetc_string = None
 
@@ -307,7 +307,7 @@ class TreeUpdater(threading.Thread):
                         meant_string = meant
                     else:
                         meant_string = "*"
-                    t_info['mean_total_elapsed_time_string'] = meant_string
+                    t_info['mean_elapsed_time_string'] = meant_string
 
                     for dt in tkeys:
                         if t_info[dt] is None:
@@ -351,7 +351,7 @@ class TreeUpdater(threading.Thread):
                     t_info['submitted_time_string'],
                     t_info['started_time_string'],
                     t_info['finished_time_string'],
-                    t_info['mean_total_elapsed_time_string'],
+                    t_info['mean_elapsed_time_string'],
                     message, icon, t_info['progress']
                 ]
                 dest[point_string][name] = new_info
