@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2016 NIWA
+# Copyright (C) 2008-2017 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -199,7 +199,11 @@ class NameExpander(object):
         if not param_list:
             # Inner loop.
             current_values = copy(spec_vals)
-            results.append((str_tmpl % current_values, current_values))
+            try:
+                results.append((str_tmpl % current_values, current_values))
+            except KeyError as exc:
+                raise ParamExpandError('ERROR: parameter %s is not '
+                                       'defined.' % str(exc.args[0]))
         else:
             for param_val in param_list[0][1]:
                 spec_vals[param_list[0][0]] = param_val
@@ -342,7 +346,11 @@ class GraphExpander(object):
                         param_values[pname] = offval
                 for pname in param_values:
                     tmpl += self.param_tmpl_cfg[pname]
-                repl = tmpl % param_values
+                try:
+                    repl = tmpl % param_values
+                except KeyError as exc:
+                    raise ParamExpandError('ERROR: parameter %s is not '
+                                           'defined.' % str(exc.args[0]))
                 line = re.sub('<' + p_group + '>', repl, line)
                 # Remove out-of-range nodes to first arrow.
                 line = re.sub('^.*--<REMOVE>--.*?=>\s*?', '', line)
@@ -529,6 +537,18 @@ class TestParamExpand(unittest.TestCase):
                           self.graph_expander.expand,
                           'foo<i=4,j><i,j>')
 
+    def test_template_fail_missing_param(self):
+        """Test a template string specifying a non-existent parameter."""
+        kvals = [str(k) for k in range(2)]
+        params_map = {'k': kvals}
+        templates = {'k': '_%(z)s'}
+        self.assertRaises(ParamExpandError,
+                          NameExpander((params_map, templates,)).expand,
+                          'foo<k>')
+        self.assertRaises(ParamExpandError,
+                          GraphExpander((params_map, templates,)).expand,
+                          'foo<k>')
+
 
 if __name__ == "__main__":
     unittest.main()
