@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2016 NIWA
+# Copyright (C) 2008-2017 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -163,20 +163,23 @@ class CylcSuiteDAO(object):
     """Data access object for the suite runtime database."""
 
     CONN_TIMEOUT = 0.2
-    DB_FILE_BASE_NAME = "cylc-suite.db"
-    PRI_DB_FILE_BASE_NAME = "cylc-suite-private.db"
-    PUB_DB_FILE_BASE_NAME = "cylc-suite-public.db"
+    DB_FILE_BASE_NAME = "db"
+    OLD_DB_FILE_BASE_NAME = "cylc-suite.db"
+    OLD_DB_FILE_BASE_NAME_611 = (
+        "cylc-suite-private.db", "cylc-suite-public.db")
     MAX_TRIES = 100
     CHECKPOINT_LATEST_ID = 0
     CHECKPOINT_LATEST_EVENT = "latest"
     TABLE_BROADCAST_EVENTS = "broadcast_events"
     TABLE_BROADCAST_STATES = "broadcast_states"
     TABLE_BROADCAST_STATES_CHECKPOINTS = "broadcast_states_checkpoints"
+    TABLE_INHERITANCE = "inheritance"
     TABLE_SUITE_PARAMS = "suite_params"
     TABLE_SUITE_PARAMS_CHECKPOINTS = "suite_params_checkpoints"
     TABLE_SUITE_TEMPLATE_VARS = "suite_template_vars"
     TABLE_TASK_JOBS = "task_jobs"
     TABLE_TASK_EVENTS = "task_events"
+    TABLE_TASK_ACTION_TIMERS = "task_action_timers"
     TABLE_CHECKPOINT_ID = "checkpoint_id"
     TABLE_TASK_POOL = "task_pool"
     TABLE_TASK_POOL_CHECKPOINTS = "task_pool_checkpoints"
@@ -209,6 +212,10 @@ class CylcSuiteDAO(object):
             ["time"],
             ["event"],
         ],
+        TABLE_INHERITANCE: [
+            ["namespace", {"is_primary_key": True}],
+            ["inheritance"],
+        ],
         TABLE_SUITE_PARAMS: [
             ["key", {"is_primary_key": True}],
             ["value"],
@@ -222,6 +229,16 @@ class CylcSuiteDAO(object):
             ["key", {"is_primary_key": True}],
             ["value"],
         ],
+        TABLE_TASK_ACTION_TIMERS: [
+            ["cycle", {"is_primary_key": True}],
+            ["name", {"is_primary_key": True}],
+            ["ctx_key_pickle", {"is_primary_key": True}],
+            ["ctx_pickle"],
+            ["delays_pickle"],
+            ["num", {"datatype": "INTEGER"}],
+            ["delay"],
+            ["timeout"],
+        ],
         TABLE_TASK_JOBS: [
             ["cycle", {"is_primary_key": True}],
             ["name", {"is_primary_key": True}],
@@ -252,6 +269,7 @@ class CylcSuiteDAO(object):
             ["name", {"is_primary_key": True}],
             ["spawned", {"datatype": "INTEGER"}],
             ["status"],
+            ["hold_swap"],
         ],
         TABLE_TASK_POOL_CHECKPOINTS: [
             ["id", {"datatype": "INTEGER", "is_primary_key": True}],
@@ -259,6 +277,7 @@ class CylcSuiteDAO(object):
             ["name", {"is_primary_key": True}],
             ["spawned", {"datatype": "INTEGER"}],
             ["status"],
+            ["hold_swap"],
         ],
         TABLE_TASK_STATES: [
             ["name", {"is_primary_key": True}],
@@ -497,6 +516,19 @@ class CylcSuiteDAO(object):
                 r"SELECT key,value FROM %s" % self.TABLE_SUITE_TEMPLATE_VARS)):
             callback(row_idx, list(row))
 
+    def select_task_action_timers(self, callback):
+        """Select from task_action_timers for restart.
+
+        Invoke callback(row_idx, row) on each row.
+        """
+        attrs = []
+        for item in self.TABLES_ATTRS[self.TABLE_TASK_ACTION_TIMERS]:
+            attrs.append(item[0])
+        stmt = r"SELECT %s FROM %s" % (
+            ",".join(attrs), self.TABLE_TASK_ACTION_TIMERS)
+        for row_idx, row in enumerate(self.connect().execute(stmt)):
+            callback(row_idx, list(row))
+
     def select_task_job(self, keys, cycle, name, submit_num=None):
         """Select items from task_jobs by (cycle, name, submit_num).
 
@@ -529,6 +561,27 @@ class CylcSuiteDAO(object):
         except sqlite3.DatabaseError:
             return None
 
+    def select_task_job_run_times(self, callback):
+        """Select run times of succeeded task jobs grouped by task names.
+
+        Invoke callback(row_idx, row) on each row, where each row contains:
+            [name, run_times_str]
+
+        where run_times_str is a string containing comma separated list of
+        integer run times. This method is used to re-populate elapsed run times
+        of each task on restart.
+        """
+        stmt = (
+            r"SELECT" +
+            r" name," +
+            r" GROUP_CONCAT(" +
+            r"     CAST(strftime('%s', time_run_exit) AS NUMERIC) -" +
+            r"     CAST(strftime('%s', time_run) AS NUMERIC))" +
+            r" FROM task_jobs" +
+            r" WHERE run_status==0 GROUP BY name ORDER BY time_run_exit")
+        for row_idx, row in enumerate(self.connect().execute(stmt)):
+            callback(row_idx, list(row))
+
     def select_task_states_by_task_ids(self, keys, task_ids=None):
         """Select items from task_states by task IDs.
 
@@ -575,7 +628,7 @@ class CylcSuiteDAO(object):
         select from task_pool table if id_key == CHECKPOINT_LATEST_ID.
         Otherwise select from task_pool_checkpoints where id == id_key.
         """
-        form_stmt = r"SELECT cycle,name,spawned,status FROM %s"
+        form_stmt = r"SELECT cycle,name,spawned,status,hold_swap FROM %s"
         if id_key is None or id_key == self.CHECKPOINT_LATEST_ID:
             stmt = form_stmt % self.TABLE_TASK_POOL
             stmt_args = []
@@ -590,7 +643,8 @@ class CylcSuiteDAO(object):
         """Select from task_pool+task_states+task_jobs for restart.
 
         Invoke callback(row_idx, row) on each row, where each row contains:
-            [cycle, name, spawned, status, submit_num, try_num, user_at_host]
+            [cycle, name, spawned, status, hold_swap, submit_num, try_num,
+             user_at_host]
 
         If id_key is specified,
         select from task_pool table if id_key == CHECKPOINT_LATEST_ID.
@@ -602,6 +656,7 @@ class CylcSuiteDAO(object):
             r"    %(task_pool)s.name," +
             r"    %(task_pool)s.spawned,"
             r"    %(task_pool)s.status," +
+            r"    %(task_pool)s.hold_swap," +
             r"    %(task_states)s.submit_num," +
             r"    %(task_jobs)s.try_num," +
             r"    %(task_jobs)s.user_at_host " +
@@ -664,6 +719,16 @@ class CylcSuiteDAO(object):
                     dao.tables[table_name + "_checkpoints"].add_insert_item(
                         [id_] + list(row))
 
+    def upgrade_from_611(self):
+        """Upgrade database on restart with a 6.11.X private database."""
+        conn = self.connect()
+        # Add hold_swap column task_pool(_checkpoints) tables
+        for t_name in [self.TABLE_TASK_POOL, self.TABLE_TASK_POOL_CHECKPOINTS]:
+            sys.stdout.write("Add hold_swap column to %s\n" % (t_name,))
+            conn.execute(
+                r"ALTER TABLE " + t_name + r" ADD COLUMN hold_swap TEXT")
+        conn.commit()
+
     def upgrade_with_state_file(self, state_file_path):
         """Upgrade database on restart with an old state file.
 
@@ -746,7 +811,8 @@ class CylcSuiteDAO(object):
             "name": name,
             "cycle": cycle,
             "spawned": spawned,
-            "status": status})
+            "status": status,
+            "hold_swap": None})
         sys.stdout.write("\n + %s" % head)
 
     def _upgrade_with_state_file_extras(self):
