@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2016 NIWA
+# Copyright (C) 2008-2017 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -96,8 +96,8 @@ def parse_commandline(is_restart):
     if is_restart:
         parser.add_option(
             "--checkpoint",
-            help="Use specified checkpoint to restart",
-            metavar="CHECKPOINT", action="store", dest="checkpoint")
+            help="Specify the ID of a checkpoint to restart from",
+            metavar="CHECKPOINT-ID", action="store", dest="checkpoint")
 
         parser.add_option(
             "--ignore-final-cycle-point",
@@ -144,9 +144,9 @@ def parse_commandline(is_restart):
 
     parser.add_option(
         "-m", "--mode",
-        help="Run mode: live, simulation, or dummy; default is live.",
+        help="Run mode: live, dummy, dummy-local, simulation (default live).",
         metavar="STRING", action="store", default='live', dest="run_mode",
-        choices=["live", "dummy", "simulation"])
+        choices=["live", "dummy", "dummy-local", "simulation"])
 
     parser.add_option(
         "--reference-log",
@@ -158,6 +158,11 @@ def parse_commandline(is_restart):
         help="Do a test run against a previously generated reference log.",
         action="store_true", default=False, dest="reftest")
 
+    parser.add_option(
+        "--source", "-S",
+        help="Specify the suite source.",
+        metavar="SOURCE", action="store", dest="source")
+
     options, args = parser.parse_args()
 
     if not is_restart and options.warm and len(args) < 2:
