@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2016 NIWA
+# Copyright (C) 2008-2017 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -439,6 +439,16 @@ class STDLogger(object):
         self.logger = logging.getLogger(log)
 
     def log(self, level, *args, **kwargs):
+        try:
+            itask = kwargs.pop("itask")
+        except KeyError:
+            pass
+        else:
+            try:
+                args = ("[%s] -%s" % (itask.identity, args[0]),) + args[1:]
+            except AttributeError:
+                args = ("[%s] -%s" % (itask, args[0]),) + args[1:]
+            args = tuple(args)
         if self.logger.handlers:
             # If this logger has file handlers write out to it.
             self.logger.log(level, *args, **kwargs)
