@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2016 NIWA
+# Copyright (C) 2008-2017 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -111,7 +111,6 @@ class CylcTimeParser(object):
     def __init__(self, context_start_point,
                  context_end_point, num_expanded_year_digits=0,
                  dump_format=None,
-                 custom_point_parse_function=None,
                  assumed_time_zone=None):
         if context_start_point is not None:
             context_start_point = str(context_start_point)
@@ -139,7 +138,6 @@ class CylcTimeParser(object):
             (re.compile(regex), msg) for (regex, msg) in
             self.POINT_INVALID_FOR_CYLC_REGEXES
         ]
-        self.custom_point_parse_function = custom_point_parse_function
         if isinstance(context_start_point, basestring):
             context_start_point, offset = self._get_point_from_expression(
                 context_start_point, None)
@@ -186,7 +184,7 @@ class CylcTimeParser(object):
                          context_start_point=None,
                          context_end_point=None):
         """Parse an expression in abbrev. or full ISO recurrence format."""
-        expression, exclusion = parse_exclusion(expression)
+        expression, exclusions = parse_exclusion(expression)
         if context_start_point is None:
             context_start_point = self.context_start_point
         if context_end_point is None:
@@ -195,6 +193,7 @@ class CylcTimeParser(object):
             result = rec_object.search(expression)
             if not result:
                 continue
+
             props = {}
             repetitions = result.groupdict().get("reps")
             if repetitions is not None:
@@ -218,13 +217,20 @@ class CylcTimeParser(object):
                 raise CylcMissingFinalCyclePointError(
                     "This suite requires a final cycle point."
                 )
+
             exclusion_point = None
-            if exclusion:
-                exclusion_point, excl_off = self._get_point_from_expression(
-                    exclusion, None, is_required=False, allow_truncated=False
-                )
-                if excl_off:
-                    exclusion_point += excl_off
+            exclusion_points = []
+            # Convert the exclusion strings to ISO8601 points
+            if exclusions is not None:
+                for exclusion in exclusions:
+                    exclusion_point, excl_off = (
+                        self._get_point_from_expression(
+                            exclusion, None, is_required=False,
+                            allow_truncated=False))
+                    if excl_off:
+                        exclusion_point += excl_off
+                    exclusion_points.append(exclusion_point)
+
             intv = result.groupdict().get("intv")
             intv_context_truncated_point = None
             if start_point is not None and start_point.truncated:
@@ -267,7 +273,7 @@ class CylcTimeParser(object):
                 start_point=start_point,
                 duration=interval,
                 end_point=end_point
-            ), exclusion_point
+            ), exclusion_points
 
         raise CylcTimeSyntaxError("Could not parse %s" % expression)
 
@@ -317,6 +323,7 @@ class CylcTimeParser(object):
 
     def _get_point_from_expression(self, expr, context, is_required=False,
                                    allow_truncated=False):
+        """Gets a TimePoint from an expression"""
         if expr is None:
             if is_required and allow_truncated:
                 if context is None:
@@ -358,11 +365,8 @@ class CylcTimeParser(object):
         expr_to_parse = expr
         if expr.endswith("T"):
             expr_to_parse = expr + "00"
-        parse_function = self.timepoint_parser.parse
-        if self.custom_point_parse_function is not None:
-            parse_function = self.custom_point_parse_function
         try:
-            expr_point = parse_function(expr_to_parse)
+            expr_point = self.timepoint_parser.parse(expr_to_parse)
         except ValueError:
             pass
         else:
@@ -372,7 +376,7 @@ class CylcTimeParser(object):
                 for rec in recs:
                     if rec.search(expr):
                         try:
-                            expr_point = parse_function(
+                            expr_point = self.timepoint_parser.parse(
                                 truncation_string + expr_to_parse)
                         except ValueError:
                             continue
