@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2016 NIWA
+# Copyright (C) 2008-2017 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,8 +20,6 @@
 from calendar import timegm
 from datetime import datetime, timedelta
 
-from isodatetime.data import Duration
-from isodatetime.parsers import TimePointParser
 from isodatetime.timezone import (
     get_local_time_zone_format, get_local_time_zone)
 
@@ -64,7 +62,7 @@ TIME_ZONE_UTC_INFO = {
     "string_extended": TIME_ZONE_STRING_UTC
 }
 
-PARSER = TimePointParser()
+PARSER = None
 
 
 def now(override_use_utc=None):
@@ -227,6 +225,10 @@ def get_unix_time_from_time_string(datetime_string):
         date_time_utc = datetime.strptime(
             datetime_string, DATE_TIME_FORMAT_EXTENDED + "Z")
     except ValueError:
+        global PARSER
+        if PARSER is None:
+            from isodatetime.parsers import TimePointParser
+            PARSER = TimePointParser()
         time_zone_info = PARSER.get_info(datetime_string)[1]
         time_zone_hour = int(time_zone_info["time_zone_hour"])
         time_zone_minute = int(time_zone_info.get("time_zone_minute", 0))
@@ -243,4 +245,5 @@ def get_unix_time_from_time_string(datetime_string):
 
 def get_seconds_as_interval_string(seconds):
     """Convert a number of seconds into an ISO 8601 duration string."""
+    from isodatetime.data import Duration
     return str(Duration(seconds=seconds, standardize=True))
