@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # THIS FILE IS PART OF THE CYLC SUITE ENGINE.
-# Copyright (C) 2008-2016 NIWA
+# Copyright (C) 2008-2017 NIWA
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -40,19 +40,11 @@ import traceback
 from parsec import ParsecError
 from parsec.OrderedDict import OrderedDictWithDefaults
 from parsec.include import inline, IncludeFileNotFoundError
+from parsec.jinja2support import jinja2process, TemplateError, UndefinedError
 from parsec.util import itemstr
 import cylc.flags
 
 
-try:
-    from parsec.jinja2support import (
-        jinja2process, TemplateError, UndefinedError)
-except ImportError:
-    jinja2_disabled = True
-else:
-    jinja2_disabled = False
-
-
 # heading/sections can contain commas (namespace name lists) and any
 # regex pattern characters (this was for pre cylc-6 satellite tasks).
 # Proper task names are checked later in config.py.
@@ -270,8 +262,6 @@ def read_and_proc(fpath, template_vars=None, viewcfg=None, asedit=False):
     # process with Jinja2
     if do_jinja2:
         if flines and re.match('^#![jJ]inja2\s*', flines[0]):
-            if jinja2_disabled:
-                raise FileParseError('Jinja2 is not installed')
             if cylc.flags.verbose:
                 print "Processing with Jinja2"
             try:
@@ -316,20 +306,16 @@ def read_and_proc(fpath, template_vars=None, viewcfg=None, asedit=False):
     return [fl.rstrip() for fl in flines]
 
 
-def parse(fpath, write_proc=False, template_vars=None):
+def parse(fpath, output_fname=None, template_vars=None):
     "Parse file items line-by-line into a corresponding nested dict."
 
     # read and process the file (jinja2, include-files, line continuation)
     flines = read_and_proc(fpath, template_vars)
-    # write the processed for suite.rc if it lives in a writable directory
-    if write_proc and \
-            os.access(os.path.dirname(fpath), os.W_OK):
-        fpath_processed = fpath + '.processed'
+    if output_fname:
+        with open(output_fname, 'wb') as handle:
+            handle.write('\n'.join(flines) + '\n')
         if cylc.flags.verbose:
-            print "Writing file " + fpath_processed
-        f = open(fpath_processed, 'w')
-        f.write('\n'.join(flines) + '\n')
-        f.close()
+            print "Processed configuration dumped: %s" % output_fname
 
     nesting_level = 0
     config = OrderedDictWithDefaults()
